
#include <tchar.h>
#include <stdio.h>
#include <string.h>
#include <stdlib.h>

#include "io/io.h"
#include "act.h"
#include "spr.h"
#include "vram.h"
#include "console.h"

struct act_spr {
	int pos_x;
	int pos_y;
	int image;
	int mirror;
	unsigned long mask;
	float scale_x;
	float scale_y;
	int rotation;
	int dontjump;
	int size_x;
	int size_y;
	int extrainfo;
	int extra_x;
	int extra_y;
};

struct act_data* act_load(struct io_data* io,const wchar_t *file) {
	char *fp;
	char *fp_base;
	int filesize;
	struct act_data *act;
	int i , j , k, magic;
	act = calloc(1,sizeof(struct act_data));
	if(act == NULL) { return NULL; }
	i = wcslen(file);
	if(i >= 3 || file[i - 3] == '.') {
		wchar_t file_spr[256];
		wcsncpy(file_spr,file,i-3);
		wcscpy(file_spr + i - 3,_T("spr"));
		act->spr = spr_load(io,file_spr);
	}
	console_outputwf(_T("    reading %s ...\n"),file);
	fp = (char*)io_read_all(io,file,&filesize);
	fp_base = fp;
	if(fp == NULL) {
		console_outputwf(_T("        File %s can't open.\n"),file);
		free(act);
		return NULL;
	}
	magic = *(short*)(fp + 2);
	act->animes = *(short *)(fp + 4);
	fp += 8;
	fp += 8; // dummy
	act->anime = (struct act_anime*)calloc(act->animes,sizeof(struct act_anime));
	if(act->anime == NULL) {
		act_free(act); free(fp_base); return NULL;
	}
	for(i = 0;i < act->animes ; i++) {
		act->anime[i].frames = *(long*)fp; fp += 4;
		act->anime[i].frame  = (struct act_frame*)calloc(act->anime[i].frames,sizeof(struct act_frame));
		if(act->anime[i].frame == NULL) {
			act_free(act); free(fp_base); return NULL;
		}
		for(j = 0; j < act->anime[i].frames; j++) {
			int sprs;
			fp += 32; // dummy
			sprs = *(long*)fp; fp += 4;
			act->anime[i].frame[j].sprs = sprs;
			act->anime[i].frame[j].spr  = (struct act_spr*)calloc(sprs,sizeof(struct act_spr));
			if(act->anime[i].frame[j].spr == NULL) {
				act_free(act); free(fp_base); return NULL;
			}
			for(k = 0;k < sprs ; k++) {
				struct act_spr *spr = &act->anime[i].frame[j].spr[k];
				spr->pos_x  = *(long*)fp; fp += 4;
				spr->pos_y  = *(long*)fp; fp += 4;
				spr->image  = *(long*)fp; fp += 4;
				spr->mirror = *(long*)fp; fp += 4;
				spr->mask   = *(unsigned long*)fp; fp += 4;
				spr->scale_x = *(float*)fp; fp += 4;
				if((magic & 0xFF) >= 0x04) {
					spr->scale_y = *(float*)fp; fp += 4;
				} else {
					spr->scale_y = spr->scale_x;
				}
				spr->rotation = *(long*)fp; fp += 4;
				spr->dontjump = *(long*)fp; fp += 4;
				if((magic & 0xFF) >= 0x05) {
					spr->size_x = *(long*)fp; fp += 4;
					spr->size_y = *(long*)fp; fp += 4;
				}
			}
			if((magic & 0xFF) >= 0x01) {
				act->anime[i].frame[j].sound = *(long*)fp; fp += 4;
			}
			if((magic & 0xFF) >= 0x02) {
				int _extrainfo , _extra_x , _extra_y;
				_extrainfo = *(long*)fp; fp += 4;
				if(_extrainfo) {
					int l;
					for(l = 0;l < _extrainfo;l++) {
						fp += 4; // dummy
						_extra_x = *(long*)fp; fp += 4;
						_extra_y = *(long*)fp; fp += 4;
						fp += 4; // dummy
					}
				}
			}
		}
	}
	if((magic & 0xFF) >= 0x01) {
		int nsounds = *(long*)fp; fp += 4;
		for(i = 0;i < nsounds;i++) {
			fp += 40;
		}
		if((magic & 0xFF) >= 0x02) {
			for(i = 0;i < act->animes;i++) {
				fp += 4;
			}
		}
	} else {
		fp += 4; // dummy
	}
	console_outputf("    done.\n");
	free(fp_base);
	return act;
}

void act_frame_draw(struct act_data *act,int anime_no,int frame_no,struct vram_data* vram,int pos_x,int pos_y) {
	struct act_frame *frame;
	int i;
	if(act == NULL || act->spr == NULL) {
		return;
	}
	frame = &act->anime[anime_no].frame[frame_no];
	for(i = 0;i < frame->sprs ; i++) {
		if(frame->spr[i].image < act->spr->frames) {
			struct vram_size from_;
			RECT from , to;
			vram_getsize(act->spr->frame[frame->spr[i].image],&from_);
			if(frame->spr[i].mirror) {
				from.left = from_.size_x - 1; from.top = 0;
				from.right = 0; from.bottom = from_.size_y - 1;
			} else {
				from.left = 0; from.top = 0;
				from.right = from_.size_x - 1; from.bottom = from_.size_y - 1;
			}
			to.left   = frame->spr[i].pos_x + pos_x - (from_.size_x) / 2;
			to.top    = frame->spr[i].pos_y + pos_y - (from_.size_y) / 2;
			to.right  = to.left + from_.size_x - 1;
			to.bottom = to.top  + from_.size_y - 1;
			vram_copy(
				act->spr->frame[frame->spr[i].image],vram,&from,&to,0,
				frame->spr[i].mask >> 24
			);
		}
	}
}

#ifndef min
#define min(a,b) ((a) < (b) ? (a) : (b))
#endif
#ifndef max
#define max(a,b) ((a) > (b) ? (a) : (b))
#endif

void act_rect_size(RECT* a,const RECT *b) {
	a->top    = min(a->top,b->top);
	a->bottom = max(a->bottom,b->bottom);
	a->left   = min(a->left,b->left);
	a->right  = max(a->right,b->right);
}

// w肵t[̑傫Ԃ
void act_frame_size(struct act_data *act,int anime_no,int frame_no,RECT *rect) {
	struct act_frame *frame;
	int i;
	if(act == NULL || act->spr == NULL) {
		rect->left   = 0;
		rect->top    = 0;
		rect->bottom = 0;
		rect->top    = 0;
		return;
	}
	frame = &act->anime[anime_no].frame[frame_no];
	rect->left   = 9999;
	rect->top    = 9999;
	rect->bottom = -9999;
	rect->right  = -9999;
	for(i = 0;i < frame->sprs ; i++) {
		if(frame->spr[i].image < act->spr->frames) {
			struct vram_size from_;
			RECT to;
			vram_getsize(act->spr->frame[frame->spr[i].image],&from_);
			to.left   = frame->spr[i].pos_x - (from_.size_x) / 2;
			to.right  = to.left + from_.size_x - 1;
			to.top    = frame->spr[i].pos_y - (from_.size_y) / 2;
			to.bottom = to.top  + from_.size_y - 1;
			act_rect_size(rect,&to);
		}
	}
}

// w肵Aj̑傫Ԃ
void act_anime_size(struct act_data *act,int anime_no,RECT *rect) {
	if(act == NULL || act->spr == NULL) {
		rect->left   = 0;
		rect->top    = 0;
		rect->bottom = 0;
		rect->top    = 0;
	} else {
		int i;
		struct act_anime *anime = &act->anime[anime_no];
		rect->left   = 9999;
		rect->top    = 9999;
		rect->bottom = -9999;
		rect->right  = -9999;
		for(i = 0;i < anime->frames ; i++) {
			RECT frame_size;
			act_frame_size(act,anime_no,i,&frame_size);
			act_rect_size(rect,&frame_size);
		}
	}
}

// act Ŝ̃TCYԂ
void act_size(struct act_data *act,RECT *rect) {
	if(act == NULL || act->spr == NULL) {
		rect->left   = 0;
		rect->top    = 0;
		rect->bottom = 0;
		rect->right  = 0;
	} else {
		int i;
		rect->left   = 9999;
		rect->top    = 9999;
		rect->bottom = -9999;
		rect->right  = -9999;
		for(i = 0;i < act->animes; i++) {
			RECT anime_size;
			act_anime_size(act,i,&anime_size);
			act_rect_size(rect,&anime_size);
		}
	}
}



// Ȃ񂾂悱̕Gȉ[`orz
void act_free(struct act_data *act) {
	int i,j;
	if(act == NULL) return;
	for(i = 0; i < act->animes ; i++) {
		if(act->anime) {
			for(j = 0; j < act->anime[i].frames ; j++) {
				if(act->anime[i].frame != NULL) {
					free(act->anime[i].frame[j].spr);
				}
			}
			free(act->anime[i].frame);
		}
	}
	free(act->anime);
	spr_free(act->spr);
	free(act);
}


void act_check(struct io_data* io,const wchar_t *file,int extra_info) {
	char *fp;
	char *fp_base;
	int filesize;
	int animes;
	int i , j , k, magic;
	fp = io_read_all(io,file,&filesize);
	fp_base = fp;
	console_outputwf(_T("Checking %s ...\n"),file);
	if(fp == NULL) {
		console_outputwf(_T("    File %s can't open.\n"),file);
		return;
	}
	magic = *(short*)(fp + 2);
	animes = *(short *)(fp + 4);
	console_outputf("    Code:%c%c Magic:%02x Objs:%d\n",fp[0],fp[1],magic,animes);
	fp += 8;
	fp += 8; // dummy
	for(i = 0;i < animes ; i++) {
		int frames,sprs;
		frames = *(long*)fp; fp += 4;
		if(extra_info) {
			console_outputf("anime %d pos:0x%x frames:%d\n",i,fp - fp_base,frames);
		}
		for(j = 0; j < frames; j++) {
			if(extra_info) {
				console_outputf("    frame:%d pos:0x%x\n",j,fp - fp_base);
			}
			fp += 32; // dummy
			sprs = *(long*)fp; fp += 4;
			for(k = 0;k < sprs ; k++) {
				struct act_spr spr_info,*spr;
				spr = &spr_info;
				spr->pos_x  = *(long*)fp; fp += 4;
				spr->pos_y  = *(long*)fp; fp += 4;
				spr->image  = *(long*)fp; fp += 4;
				spr->mirror = *(long*)fp; fp += 4;
				spr->mask   = *(unsigned long*)fp; fp += 4;
				spr->scale_x = *(float*)fp; fp += 4;
				if((magic & 0xFF) >= 0x04) {
					spr->scale_y = *(float*)fp; fp += 4;
				} else {
					spr->scale_y = spr->scale_x;
				}
				spr->rotation = *(long*)fp; fp += 4;
				spr->dontjump = *(long*)fp; fp += 4;
				if((magic & 0xFF) >= 0x05) {
					spr->size_x = *(long*)fp; fp += 4;
					spr->size_y = *(long*)fp; fp += 4;
				}
				if(extra_info) {
					console_outputf(
						"        spr:%d pos(%d,%d) img:%d mirror:%d\n",
						k,spr->pos_x,spr->pos_y,spr->image,spr->mirror
					);
				}
			}
			if(extra_info) {
				console_outputf("    frame:%d",j);
			}
			if((magic & 0xFF) >= 0x01) {
				int sound = *(long*)fp; fp += 4;
				if(extra_info) {
					console_outputf(" sound: %d",sound);
				}
			}
			if((magic & 0xFF) >= 0x02) {
				int _extrainfo , _extra_x , _extra_y;
				_extrainfo = *(long*)fp; fp += 4;
				if(extra_info) {
					console_outputf(" extra:%d ",_extrainfo);
				}
				if(_extrainfo) {
					int l;
					if(extra_info) {
						console_outputf("[");
					}
					for(l = 0;l < _extrainfo;l++) {
						fp += 4; // dummy
						_extra_x = *(long*)fp; fp += 4;
						_extra_y = *(long*)fp; fp += 4;
						fp += 4; // dummy
						if(extra_info) {
							console_outputf("%d,%d ",_extra_x,_extra_y);
						}
					}
					if(extra_info) {
						console_outputf("]");
					}
				}
			}
			if(extra_info) {
				console_outputf("\n");
			}
		}
	}
	if((magic & 0xFF) >= 0x01) {
		int nsounds = *(long*)fp; fp += 4;
		if(extra_info) {
			console_outputf("sounds: %d pos:0x%x\n",nsounds,fp - fp_base - 2);
		}
		for(i = 0;i < nsounds;i++) {
			if(extra_info) {
				console_outputf("    sound %d : %s\n",i,fp);
			}
			fp += 40;
		}
		if((magic & 0xFF) >= 0x02) {
			if(extra_info) {
				console_outputf("intarvels: %d\n",animes);
			}
			for(i = 0;i < animes;i++) {
				if(extra_info) {
					console_outputf("intarvel %d: 25 * %fms(%fms)\n",i,*(float*)fp,25 * *(float*)fp);
				}
				fp += 4;
			}
		}
	} else {
		fp += 4; // dummy
	}
	if(fp - fp_base != filesize) {
		console_outputf("    ERR: Left %d Byte\n",filesize - (fp - fp_base));
	} else {
		console_outputf("    Read Ok 0x%x\n",fp - fp_base);
	}
	free(fp_base);
}
