
#include <tchar.h>
#include <stdio.h>
#include <stdlib.h>

#include "io/io.h"
#include "gat.h"
#include "gmap.h"
#include "console.h"

typedef struct {
	struct gmap_height height;
	int type;
} _gat_cell;

void gat_output_map(struct gmap_data* map,struct io_data* io,wchar_t *file_pre) {
	short temp_s;
	long  temp_l;
	wchar_t  file[256];
	struct io_fp *fp;
	int   x,y;
	_gat_cell cell;
	struct gmap_height height;

	if(map == NULL) { return; }
	swprintf(file,_T("%s.gat"),file_pre);
	console_outputwf(_T("    writing %s\n"),file);

	fp = io_write_open(io,file);
	if(fp == NULL) { return; }
	io_write(fp,"GRAT",4);
	// magic
	temp_s = 0x0201;
	io_write(fp,&temp_s,sizeof(short));

	// size_x
	temp_l = map->map_width * 2;
	io_write(fp,&temp_l,sizeof(int));

	// size_y
	temp_l = map->map_height * 2;
	io_write(fp,&temp_l,sizeof(int));

	memset(&cell,0,sizeof(cell));
	// ɂQ{ɂďo͂
	for(y = map->map_height - 1; y  >= 0 ; y--) {
		for(x = 0;x < map->map_width ; x++) {
			if(map->movable) {
				if(map->cfg_flag_expert) {
					cell.type = map->movable->cell[(y*2) * (map->map_width*2) + (x*2)] ? 1 : 0;
				} else {
					cell.type = map->movable->cell[y * map->map_width + x] ? 1 : 0;
				}
			} else {
				cell.type = 0;
			}
			gmap_height(map,&height,x,y);

			// 
			cell.height.z_lt = (height.z_lt + height.z_lb) / 2;
			cell.height.z_lb = height.z_lb;
			cell.height.z_rt = (height.z_lt + height.z_lb + height.z_rt + height.z_rb) / 4;
			cell.height.z_rb = (height.z_rb + height.z_lb) / 2;
			io_write(fp,&cell,sizeof(_gat_cell));

			// E
			cell.height.z_lt = (height.z_lt + height.z_lb + height.z_rt + height.z_rb) / 4;
			cell.height.z_lb = (height.z_rb + height.z_lb) / 2;
			cell.height.z_rt = (height.z_rt + height.z_rb) / 2;
			cell.height.z_rb = height.z_rb;
			io_write(fp,&cell,sizeof(_gat_cell));
		}
		for(x = 0;x < map->map_width ; x++) {
			if(map->movable) {
				if(map->cfg_flag_expert) {
					cell.type = map->movable->cell[(y*2) * (map->map_width*2) + (x*2)] ? 1 : 0;
				} else {
					cell.type = map->movable->cell[y * map->map_width + x] ? 1 : 0;
				}
			} else {
				cell.type = 0;
			}
			gmap_height(map,&height,x,y);
			// 㕔
			cell.height.z_lt = height.z_lt;
			cell.height.z_lb = (height.z_lt + height.z_lb) / 2;
			cell.height.z_rt = (height.z_lt + height.z_rt) / 2;
			cell.height.z_rb = (height.z_lt + height.z_lb + height.z_rt + height.z_rb) / 4;
			io_write(fp,&cell,sizeof(_gat_cell));

			// E㕔
			cell.height.z_lt = (height.z_lt + height.z_rt) / 2;
			cell.height.z_rt = height.z_rt;
			cell.height.z_lb = (height.z_lt + height.z_lb + height.z_rt + height.z_rb) / 4;
			cell.height.z_rb = (height.z_rt + height.z_rb) / 2;
			io_write(fp,&cell,sizeof(_gat_cell));
		}
	}
	io_close(fp);
}

void gat_check(struct io_data* io,const wchar_t *file,int extra_info) {
	char *fp;
	char *fp_base;
	int magic;
	int filesize;
	int x,y;
	int size_x;
	int size_y;
	unsigned char buf[256];
	_gat_cell cell;
	fp = (char*)io_read_all(io,file,&filesize);
	fp_base = fp;
	console_outputf("Checking %s ...\n",file);
	if(fp == NULL) {
		console_outputwf(_T("    File %s can't open.\n"),file);
		return;
	}
	memcpy(buf,fp,6); fp += 6;
	magic = *(unsigned short *)(buf + 4);
	buf[4] = 0;
	memcpy(&size_x,fp,sizeof(int)); fp += sizeof(int);
	memcpy(&size_y,fp,sizeof(int)); fp += sizeof(int);
	console_outputf("    Code:%s Magic:%02x Size:(%d,%d)\n",buf,magic,size_x,size_y);
	for(y = 0;y < size_y ; y++) {
		for(x = 0 ; x < size_x; x++) {
			memcpy(&cell,fp,sizeof(_gat_cell)); fp += sizeof(_gat_cell);
		}
	}

	if(filesize != fp - fp_base) {
		console_outputf("    ERR: Left %d Byte\n",filesize - (fp - fp_base));
	} else {
		console_outputf("    Read Ok 0x%x\n",fp - fp_base);
	}
	free(fp_base);
}

