// gnd.cpp : R\[ AvP[Vp̃Gg |Cg̒`
//

#include <stdio.h>
#include <tchar.h>
#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <windows.h>

#include "gmap.h"
#include "vram.h"
#include "io/io.h"
#include "gnd.h"
#include "console.h"

typedef unsigned char _gnd_lightmap[256];

typedef struct {
	float u1;
	float u2;
	float u3;
	float u4;
	float v1;
	float v2;
	float v3;
	float v4;

	unsigned short text;
	unsigned short lmap;

	int todo;
} _gnd_tile;

typedef struct {
	struct gmap_height height;
	int tilesup;
	int tileotherside;
	int tileside;
} _gnd_cube;

typedef struct {
	char filecode[4];
	short magicnumber;
} _gnd_header;

struct gnd_info {
	int sizeX;
	int sizeY;
	int ntextures;
	int ntiles;
	int nlightmaps;

	_gnd_cube *cubes;
	_gnd_tile *tiles;
	_gnd_header header;
	_gnd_lightmap *lightmaps;
	struct vram_data** texture_data;
	struct vram_size*  texture_size;
	char  *buf;
};

void gnd_output_bmp_tile(struct gnd_info *info,struct vram_data *vram,_gnd_tile *tile,int x,int y,int L,int angle,int grid);

struct gnd_info* gnd_load(struct io_data* io,const wchar_t *file) {
	int i;
	char *fp;
	wchar_t buf[256];
	struct gnd_info *info = (struct gnd_info *)calloc(sizeof(struct gnd_info),1);
	console_outputwf(_T("    reading %s ...\n"),file);
	if(info == NULL) {
		return NULL;
	}

	fp = (char*)io_read_all(io,file,NULL);
	if(fp == NULL) {
		console_outputf("    File %s can't open.\n",file);
		free(info);
		return NULL;
	}
	info->buf = fp;
	memcpy(&info->header,fp,sizeof(_gnd_header));
	fp += sizeof(_gnd_header);
	if(memcmp(info->header.filecode,"GRGN",4)) {
		console_outputf("    unsupported file : %s\n",file);
		gnd_free(info);
		return NULL;
	}

	memcpy(&info->sizeX,fp,sizeof(int)); fp += sizeof(int);
	memcpy(&info->sizeY,fp,sizeof(int)); fp += sizeof(int);
	fp += 4;

	memcpy(&info->ntextures,fp,sizeof(int)); fp += sizeof(int);
	fp += 4;
	info->texture_data = (struct vram_data**)malloc(info->ntextures * sizeof(struct vram_data*));
	info->texture_size = (struct vram_size*) malloc(info->ntextures * sizeof(struct vram_size));
	console_outputf("        reading textures(%d)... ",info->ntextures);
	for (i = 0; i < info->ntextures; i++) {
		unsigned char *mem;
		const wchar_t* pos = wcsrchr(file,'\\');
		wchar_t texture_name[128]; 
		if(pos == NULL) pos = wcsrchr(file,'/');
		io_mbcs_to_unicode(io,texture_name,fp);
		if(pos != NULL) {
			wcsncpy(buf,file,pos - file + 1);
			swprintf(buf + (pos - file) + 1,_T("texture\\%s"),texture_name);
		} else {
			swprintf(buf,_T("texture\\%s"),texture_name);
		}
		fp+=80;
		mem = (unsigned char*)io_read_all(io,buf,NULL);

		info->texture_data[i] = vram_bmpload_mem(mem);
		vram_getsize(info->texture_data[i],&info->texture_size[i]);
		if(mem == NULL) {
			console_outputwf(_T("x : %s\n"),buf);
		} else {
			console_outputf("o");
		}
		free(mem);
	}

	memcpy(&info->nlightmaps,fp,sizeof(int)); fp += sizeof(int);
	fp += 12;

	info->lightmaps = (_gnd_lightmap*)fp;
	fp += sizeof(_gnd_lightmap) * info->nlightmaps;

	memcpy(&info->ntiles,fp,sizeof(int)); fp += sizeof(int);
	info->tiles = (_gnd_tile *)fp;
	fp += sizeof(_gnd_tile) * info->ntiles;

	info->cubes = (_gnd_cube *)fp;
	// fp += sizeof(_gnd_cube) * info->sizeX * info->sizeY;
	console_outputf("\n    done.\n");
	return info;
}

void gnd_check(struct io_data* io,const wchar_t *file,int extra_info) {
	int i;
	int size;
	char *fp;
	char buf[256];
	struct gnd_info *info = (struct gnd_info *)calloc(sizeof(struct gnd_info),1);
	if(info == NULL) {
		return;
	}
	console_outputf("Checking %s ...\n",file);

	fp = (char*)io_read_all(io,file,&size);
	if(fp == NULL) {
		free(info);
		console_outputf("    File %s can't open.\n",file);
		return;
	}
	info->buf = fp;
	memcpy(&info->header,fp,sizeof(_gnd_header));

	fp += sizeof(_gnd_header);
	if(memcmp(info->header.filecode,"GRGN",4)) {
		console_outputf("gnd_check unsupported file : %s\n",file);
		free(info);
		return;
	}

	console_outputf(
		"    Code:%c%c%c%c Magic:%02x\n",
		info->header.filecode[0],info->header.filecode[1],info->header.filecode[2],
		info->header.filecode[3],info->header.magicnumber
	);

	memcpy(&info->sizeX,fp,sizeof(int)); fp += sizeof(int);
	memcpy(&info->sizeY,fp,sizeof(int)); fp += sizeof(int);
	fp += 4;

	memcpy(&info->ntextures,fp,sizeof(int)); fp += sizeof(int);
	fp += 4;

	info->texture_data = (struct vram_data**)calloc(info->ntextures,sizeof(struct vram_data*));
	info->texture_size = (struct vram_size*) calloc(info->ntextures,sizeof(struct vram_size));
	for (i = 0; i < info->ntextures; i++) {
		sprintf(buf,"texture\\%s",fp); fp+=80;
		if(extra_info) {
			console_outputf("    texture[%d] : %s\n",i,buf);
		}
	}

	/* lightmaps */
	memcpy(&info->nlightmaps,fp,sizeof(int)); fp += sizeof(int);
	fp += 12;
	if(extra_info) {
		console_outputf("    lightmaps : %d\n",info->nlightmaps);
	}

	info->lightmaps = (_gnd_lightmap*)fp;
	fp += sizeof(_gnd_lightmap) * info->nlightmaps;

	/* tiles */
	memcpy(&info->ntiles,fp,sizeof(int)); fp += sizeof(int);
	if(extra_info) {
		console_outputf("    tiles    : %d\n",info->ntiles);
	}

	info->tiles = (_gnd_tile *)fp;
	fp += sizeof(_gnd_tile) * info->ntiles;

	/* cubes */
	info->cubes = (_gnd_cube *)fp;
	fp += sizeof(_gnd_cube) * info->sizeX * info->sizeY;
	if(size != fp - info->buf) {
		console_outputf("    ERR: Left %d Byte\n",size - (fp - info->buf));
	} else {
		console_outputf("    Read Ok 0x%x\n",fp - info->buf);
	}
	gnd_free(info);
	return;
}

void gnd_free(struct gnd_info *info) {
	int i;
	if(info == NULL) { return; }
	for(i = 0; i < info->ntextures ; i++) {
		vram_free(info->texture_data[i]);
		info->texture_data[i] = NULL;
	}
	free(info->texture_data);	info->texture_data = NULL;
	free(info->texture_size);	info->texture_size = NULL;
	free(info->buf);			info->buf = NULL;
	free(info);
}

void gnd_output_bmp(struct gnd_info *info,struct io_data* io,wchar_t *filename,int L,int angle,int grid,int depth) {
	int x,y;
	struct vram_data *vram;
	if(info == NULL) { return; }
	vram = vram_alloc(info->sizeX*L,L);
	vram_bmpsave_start(vram,io,filename,info->sizeY * L,depth);
	console_outputwf(_T("    writing %s... rows:%d"),filename,info->sizeY);
	for (y = 0; y < info->sizeY ; y++) {
		if(y % 50 == 0) {
			console_outputf("\n        ");
		}
		console_outputf("o");
		for (x = 0; x < info->sizeX; x++) {
			int tilesup;
			int tileside;
			int tileotherside;

			tilesup = info->cubes[y*info->sizeX+x].tilesup;
			tileside = info->cubes[y*info->sizeX+x].tileside;
			tileotherside = info->cubes[y*info->sizeX+x].tileotherside;
			if (tileotherside != -1) {
				// gnd_output_bmp_tile(info,vram,&info->tiles[tileotherside],x,0,L,angle,grid);
			}
			if (tileside != -1) {	
				// gnd_output_bmp_tile(info,vram,&info->tiles[tileside],x,0,L,angle,grid);
			}
			if (tilesup != -1 && info->texture_data[info->tiles[tilesup].text] != NULL) {
				gnd_output_bmp_tile(info,vram,&info->tiles[tilesup],x,0,L,angle,grid);
			} else {
				vram_square(vram,x*L,0,(x+1)*L-1,L-1,vram_RGB(255,0,255));
			}
		}
		vram_bmpsave_data(vram,L);
	}
	vram_bmpsave_end(vram);
	vram_free(vram);
	console_outputf("\n    done.\n");
}

void gnd_output_bmp_tile(struct gnd_info *info,struct vram_data* vram,_gnd_tile *tile,int x,int y,int L,int angle_flag,int grid) {
	RECT from;
	RECT to;
	int angle;
	if(tile->u1 > 1 || tile->v1 > 1 || tile->u3 > 1 || tile->v3 > 1) {
		return;
	}
	if(tile->u1 == tile->u2 && tile->u3 == tile->u4 && tile->v1 == tile->v3 && tile->v2 == tile->v4) {
		angle = 90;
		from.left   = (int)(tile->u1 * info->texture_size[tile->text].size_x);
		from.right  = (int)(tile->u3 * info->texture_size[tile->text].size_x);
		from.top    = (int)(tile->v1 * info->texture_size[tile->text].size_y);
		from.bottom = (int)(tile->v2 * info->texture_size[tile->text].size_y);
	} else if(tile->u1 == tile->u3 && tile->u2 == tile->u4 && tile->v1 == tile->v2 && tile->v3 == tile->v4) {
		angle = 0;
		from.left   = (int)(tile->u1 * info->texture_size[tile->text].size_x);
		from.right  = (int)(tile->u2 * info->texture_size[tile->text].size_x);
		from.top    = (int)(tile->v3 * info->texture_size[tile->text].size_y);
		from.bottom = (int)(tile->v1 * info->texture_size[tile->text].size_y);
	} else {
		return;
	}

	if(from.top > from.bottom) {
		from.top--;
	} else {
		from.bottom--;
	}
	if(from.left > from.right) {
		from.left--;
	} else {
		from.right--;
	}
	to.left = x * L; to.right  = (x + 1) * L -1;
	to.top = y * L;  to.bottom = (y + 1) * L -1;

	vram_copy(info->texture_data[tile->text],vram,&from,&to,angle,255);
	if(grid) {
		vram_setpixel(vram,x*L,y*L,vram_RGB(0xFF,0xFF,0x00));
		vram_setpixel(vram,x*L,(y+1)*L-1,vram_RGB(0xFF,0xFF,0x00));
		vram_setpixel(vram,(x+1)*L-1,y*L,vram_RGB(0xFF,0xFF,0x00));
		vram_setpixel(vram,(x+1)*L-1,(y+1)*L-1,vram_RGB(0xFF,0xFF,0x00));
	}
	if(angle_flag) {
		char an[16];
		sprintf(an,"%d",angle + (from.top > from.bottom ? 180 : 0));
		vram_drawtext(vram,an,x*L+1,y*L+1,vram_RGB(255,0,0));
	}
}

void gnd_output_map_tile(struct gmap_data* map,int cell_dat,_gnd_tile *tile,struct io_fp *fp) {
	int x = cell_dat & 0xFF;
	int y = (cell_dat & 0xFF00) >> 8;
	int lay = cell_dat >> 16;
	float dx = (float)1 / map->layer[lay].tiles_x;
	float dy = (float)1 / map->layer[lay].tiles_y;
	if(map->cfg_cut_texture) {
		y--;
	}
	tile->u1 = tile->u3 = dx * x;
	tile->u2 = tile->u4 = dx * (x + 1);
	tile->v1 = tile->v2 = dy * (y + 1);
	tile->v3 = tile->v4 = dy * y;
	tile->text = map->layer[lay].active_layer_index;
	io_write(fp,tile,sizeof(*tile));
}

void gnd_output_map(struct gmap_data* map,struct io_data *io,wchar_t *file_pre) {
	int   i,j;
	short temp_s;
	long  temp_l;
	char  blank[256];
	wchar_t  file[256];
	int   tiles_pos, tiles_count;
	_gnd_tile tile;
	_gnd_cube cell;
	_gnd_lightmap lightmap;
	struct io_fp *fp;
	if(map == NULL) {
		return;
	}
	swprintf(file,_T("%s.gnd"),file_pre);
	console_outputwf(_T("    writing %s\n"),file);

	fp = io_write_open(io,file);

	if(fp == NULL) {
		return;
	}
	memset(blank,0,256);
	io_write(fp,"GRGN",4);
	temp_s = 0x0701;
	io_write(fp,&temp_s,sizeof(short));
	io_write(fp,&map->map_width,sizeof(int));
	io_write(fp,&map->map_height,sizeof(int));
	io_write(fp,blank,2);

	// ̂0x4120
	temp_s = 0x4120;
	io_write(fp,&temp_s,sizeof(short));

	io_write(fp,&map->active_layers,sizeof(int));

	// ̂ 0x50 
	temp_l = 0x50;
	io_write(fp,&temp_l,4);

	for(i = 0;i < map->layer_count; i++) {
		if(&map->layer[i] == map->movable || &map->layer[i] == map->height) {
			;
		} else {
			temp_l = strlen(map->layer[i].name_textureA);
			io_write(fp,map->layer[i].name_textureA,temp_l);
			io_write(fp,blank,80 - temp_l);
		}
	}

	// lightmaps : 1
	temp_l = 1;
	io_write(fp,&temp_l,sizeof(int));
	io_write(fp,blank,12);
	memset(&lightmap,0xFF,64);
	memset(lightmap+64,0x00,sizeof(lightmap) - 64);
	io_write(fp,&lightmap,sizeof(lightmap));

	// tile ̍ėp͏oȂƂӂdlɂȂĂ̂ŁA
	// tile }bvTCY쐬ȂႢȂƂ㩁B

	// tiles
	tiles_pos   = io_tell(fp);
	tiles_count = 0;
	i = map->map_height * map->map_width;
	io_write(fp,&i,sizeof(int));
	memset(&tile,0,sizeof(tile));
	tile.lmap = 0;
	tile.todo = -1;

	for(i = map->map_height - 1 ; i >= 0 ; i--) {
		for(j = 0; j < map->map_width ; j++) {
			if(map->cfg_flag_expert) {
				int pos = i * map->map_width * 4 + j*2;
				if(map->cells[pos]) {
					gnd_output_map_tile(map,map->cells[pos],&tile,fp);
					tiles_count++;
				}
				if(map->cells[pos + 1]) {
					gnd_output_map_tile(map,map->cells[pos + 1],&tile,fp);
					tiles_count++;
				}
				if(i > 0 && map->cells[pos - map->map_width * 2]) {
					gnd_output_map_tile(map,map->cells[pos - map->map_width * 2],&tile,fp);
					tiles_count++;
				}
			} else {
				int pos = i * map->map_width + j;
				if(map->cells[pos]) {
					gnd_output_map_tile(map,map->cells[pos],&tile,fp);
					tiles_count++;
				}
			}
		}
	}
	// console_outputf("gnd_output_map: tiles_count: %d\n",tiles_count);

	// cells
	tiles_count = 0;
	memset(&cell,0,sizeof(cell));

	for(i = map->map_height - 1 ; i >= 0 ; i--) {
		for(j = 0; j < map->map_width ; j++) {
			if(map->cfg_flag_expert) {
				int pos = i * map->map_width * 4 + j*2;
				if(map->cells[pos]) {
					cell.tilesup = tiles_count++;
				} else {
					cell.tilesup = -1;
				}
				if(map->cells[pos + 1]) {
					cell.tileside = tiles_count++;
				} else {
					cell.tileside = -1;
				}
				if(i > 0 && map->cells[pos - map->map_width * 2]) {
					cell.tileotherside = tiles_count++;
				} else {
					cell.tileotherside = -1;
				}
			} else {
				int pos = i * map->map_width + j;
				cell.tileside      = -1;
				cell.tileotherside = -1;
				if(map->cells[pos]) {
					cell.tilesup = tiles_count++;
				} else {
					cell.tilesup = -1;
				}
			}
			gmap_height(map,&cell.height,j,i);
			io_write(fp,&cell,sizeof(cell));
		}
	}
	io_seek(fp,tiles_pos);
	io_write(fp,&tiles_count,sizeof(int));
	// console_outputf("gnd_output_map: tiles_count: %d\n",tiles_count);

	io_close(fp);

}

#if 0
	// ^CėpoƎvōĖvɂȂo̓[`
	// }bv\ȂȏɂȂāAĂB
	// ɂĂ͖Ȃ̂ŋCɂȂłB

	// tiles
	fwrite(&map->tiles,1,sizeof(int),fp);
	memset(&tile,0,sizeof(tile));
	tile.lmap = 0;
	for(i = 0;i < map->layer_count; i++) {
		if(&map->layer[i] == map->movable || &map->layer[i] == map->height) {
			;
		} else {
			float x,y;
			float dx = (float)1 / map->layer[i].tiles_x;
			float dy = (float)1 / map->layer[i].tiles_y;
			tile.text = i;
			tile.todo = -1;
			for(y = 0 ; y < 1 ; y += dy) {
				for(x = 0 ; x < 1 ; x += dx) {
					// angle == 0
					tile.u1 = tile.u3 = x;
					tile.u2 = tile.u4 = x + dx;
					tile.v1 = tile.v2 = y + dy;
					tile.v3 = tile.v4 = y;
					fwrite(&tile,1,sizeof(tile),fp);
				}
			}
		}
	}

	// cells
	memset(&cell,0,sizeof(cell));
	cell.tileotherside = -1;
	cell.tileside      = -1;
	for(i = 0 ; i < map->map_height ; i++) {
		int pos = (map->map_height - i - 1) * map->map_width;
		int j;
		for(j = 0; j < map->map_width ; j++) {
			if(map->cells[pos]) {
				int cell_no = map->cells[pos];
				cell.tilesup = 
					map->layer[cell_no >> 16].tiles_start +
					((cell_no & 0xFF00) >> 8) * map->layer[cell_no >> 16].tiles_x +
					cell_no & 0xFF;
			} else {
				cell.tilesup = -1;
			}
			fwrite(&cell,1,sizeof(cell),fp);
			pos++;
		}
	}
#endif