// igun.cpp : R\[ AvP[Vp̃Gg |Cg̒`
//

#include <tchar.h>
#include <stdio.h>
#include "gmap.h"
#include "io/io.h"
#include "spr.h"
#include "act.h"
#include "gnd.h"
#include "rsw.h"
#include "rsw_edit.h"
#include "gat.h"
#include "igun.h"
#include "md5calc.h"
#include "client.h"
#include "console.h"

#if defined(_DEBUG) && defined(_MSC_VER)
#include <CRTDBG.H>
void debug_mem(void) {
	console_outputf("checking memory leaks...\n");
	_CrtDumpMemoryLeaks();
	console_outputf("done.");
}
#endif

enum {
	MODE_GMAP = 1 , MODE_GND , MODE_CHECK , MODE_SPR , MODE_INFO , MODE_ACT ,
	MODE_RSW , MODE_PPJCONV , MODE_EXT
};

void igun_exec(struct io_data* io,const wchar_t *expr,struct igun_config* cfg);
void igun_usage(struct igun_config* cfg);

void igun_exec_gnd(struct io_data* io,const wchar_t *file,struct igun_config *cfg);
void igun_exec_gmap(struct io_data* io,const wchar_t *file,struct igun_config *cfg);
void igun_exec_check(struct io_data* io,const wchar_t *file,struct igun_config *cfg);
void igun_exec_spr(struct io_data* io,const wchar_t *file,struct igun_config *cfg);
void igun_exec_info(struct io_data* io,const wchar_t *file,struct igun_config *cfg);
void igun_exec_info_fin(struct io_data* io,struct igun_config *cfg);
void igun_exec_act(struct io_data* io,const wchar_t *file,struct igun_config *cfg);
void igun_exec_rsw(struct io_data* io,const wchar_t *file,struct igun_config *cfg);
void igun_exec_ext(struct io_data* io,const wchar_t *file,struct igun_config *cfg);
void igun_exec_ppjconv(struct io_data* io,const wchar_t *file,struct igun_config *cfg);

int main(void)
{
	int       argc;
	wchar_t** argv;
	int    mode = 0;
	struct igun_config cfg;
	struct io_data *io;

	argv = CommandLineToArgvW(GetCommandLineW(),&argc);

	memset(&cfg,0,sizeof(struct igun_config));
	cfg.gnd_zoom = 16;
	cfg.depth    = 16;
	cfg.locate   = GetUserDefaultLangID() & 0x03FF;

#if defined(_DEBUG) && defined(_MSC_VER)
	_CrtSetReportMode(_CRT_WARN, _CRTDBG_MODE_FILE);
	_CrtSetReportFile(_CRT_WARN, _CRTDBG_FILE_STDOUT);
	_CrtSetReportMode(_CRT_ERROR, _CRTDBG_MODE_FILE);
	_CrtSetReportFile(_CRT_ERROR, _CRTDBG_FILE_STDOUT);
	_CrtSetReportMode(_CRT_ASSERT, _CRTDBG_MODE_FILE);
	_CrtSetReportFile(_CRT_ASSERT, _CRTDBG_FILE_STDOUT);

	_CrtSetDbgFlag(_CRTDBG_LEAK_CHECK_DF | _CRTDBG_REPORT_FLAG);
	atexit(debug_mem);
#endif

	io_init();
	io = io_create();

	if(argc == 1) {
		char code;
		console_outputf("    fobOpNCAgN܂H(y/n)");
		code = getchar();
		if(code == 'y' || code == 'Y') {
			client_main();
		} else {
			igun_usage(&cfg);
		}
	} else {
		int i;
		for(i = 1 ; i < argc ; i++) {
			if(argv[i][0] == '-') {
				if(!wcscmp(&argv[i][1],_T("-angle"))) {
					cfg.gnd_angle = 1;
				} else if(!wcscmp(&argv[i][1],_T("-grid"))) {
					cfg.gnd_grid = 1;
				} else if(!wcscmp(&argv[i][1],_T("-md5"))) {
					cfg.info_md5  = 1;
				} else if(!wcscmp(&argv[i][1],_T("-size"))) {
					cfg.info_size = 1;
				} else if(!wcscmp(&argv[i][1],_T("-extra"))) {
					cfg.check_extra = 1;
				} else if(!wcscmp(&argv[i][1],_T("-depth"))) {
					if(argv[i + 1]) {
						cfg.depth = _wtoi(argv[++i]);
						if(cfg.depth != 16 && cfg.depth != 24 && cfg.depth != 32) {
							cfg.depth = 16;
						}
					}
				} else if(!wcscmp(&argv[i][1],_T("-limit"))) {
					if(argv[i + 1]) {
						cfg.limit = _wtoi(argv[++i]);
						if(cfg.limit < 0) { cfg.limit = 0; }
					}
				} else if(!wcscmp(&argv[i][1],_T("-zoom"))) {
					if(argv[i + 1]) {
						cfg.gnd_zoom = _wtoi(argv[++i]);
						if(cfg.gnd_zoom < 0 || cfg.gnd_zoom > 64) { cfg.gnd_zoom = 16; }
					}
				} else if(!wcscmp(&argv[i][1],_T("-grf"))) {
					if(argv[i + 1]) { io_read_grf(io,argv[++i]); }
				} else if(!wcscmp(&argv[i][1],_T("map"))) {
					cfg.mode = MODE_GMAP;
				} else if(!wcscmp(&argv[i][1],_T("gnd"))) {
					cfg.mode = MODE_GND;
				} else if(!wcscmp(&argv[i][1],_T("check"))) {
					cfg.mode = MODE_CHECK;
				} else if(!wcscmp(&argv[i][1],_T("spr"))) {
					cfg.mode = MODE_SPR;
				} else if(!wcscmp(&argv[i][1],_T("info"))) {
					cfg.mode = MODE_INFO;
				} else if(!wcscmp(&argv[i][1],_T("act"))) {
					cfg.mode = MODE_ACT;
				} else if(!wcscmp(&argv[i][1],_T("rsw"))) {
					cfg.mode = MODE_RSW;
				} else if(!wcscmp(&argv[i][1],_T("ppj"))) {
					cfg.mode = MODE_PPJCONV;
				} else if(!wcscmp(&argv[i][1],_T("ext"))) {
					cfg.mode = MODE_EXT;
				} else if(!wcscmp(&argv[i][1],_T("client"))) {
					client_main();
				} else {
					console_outputwf(_T("Unknown option:%s\n"),argv[i]);
				}
			} else {
				igun_exec(io,argv[i++],&cfg);
			}
		}
	}
	io_delete(io);
	io_final();
	GlobalFree(argv);
	return 0;
}

void igun_usage(struct igun_config* cfg) {
	console_outputf("igun v0.12 (language: %s)\n",cfg->locate == LANG_JAPANESE ? "jp" : "en");
	console_outputf("\n");
	console_outputf("Usage: igun [command] [option] [files]\n");
	console_outputf("\n");
	console_outputf("Commands:\n");
	console_outputf("    -map   [expr]    : [ppj] }bvt@CɃRo[g\n");
	console_outputf("    -gnd   [expr]    : [gnd] rbg}bvɕϊ\n");
	console_outputf("    -check [expr]    : [gnd], [gat], [rsw] ̌\n");
	console_outputf("    -spr   [expr]    : [spr] rbg}bvɕϊ\n");
	console_outputf("    -act   [expr]    : [act] avi ɕϊRo[gibj\n");
	console_outputf("    -rsw   [expr]    : [rsw] ̕ҏW\n");
	console_outputf("    -ppj   [expr]    : [ppj] GLXp[gdlɕϊ\n");
	console_outputf("    -info  [expr]    : t@C̏\\n");
	console_outputf("    -ext   [expr]    : t@CWJ\n");
	console_outputf("    -client          : fobOpNCAgN\n");
	console_outputf("\n");
	console_outputf("Options:\n");
	console_outputf("    --grf   [file]  : grf t@CǂݍށB\n");
	console_outputf("    --limit [count] : t@C擪[count] B\n");
	console_outputf("    --depth [count] : o̓rbg}bv16/24/32 bpiɂBW:16\n");
	console_outputf("  -gnd ̂ݗL\n");
	console_outputf("    --zoom [SIZE]   : P}Xsize x size hbgŏóBW:16\n");
	console_outputf("    --angle         : px̏o\n");
	console_outputf("    --grid          : }X̋؂̒ǉ\n");
	console_outputf("  -info ̂ݗL\n");
	console_outputf("    --md5           : md5 vZ\n");
	console_outputf("  -check ̂ݗL\n");
	console_outputf("    --extra         : ڍ׏\ifobOpj\n");
}

void igun_exec(struct io_data* io,const wchar_t *expr,struct igun_config* cfg) {
	struct io_find *find = io_find_start(io,expr);
	const wchar_t *file;
	int limit = (cfg->limit ? cfg->limit + 1 : 0);
	while((file = io_find_next(find)) && limit-- != 1) {
		switch(cfg->mode) {
		case 0: console_outputwf(_T("R}hw肵Ă : %s\n"),file); break;
		case MODE_GND:     igun_exec_gnd(io,file,cfg); break;
		case MODE_GMAP:    igun_exec_gmap(io,file,cfg); break;
		case MODE_CHECK:   igun_exec_check(io,file,cfg); break;
		case MODE_SPR:     igun_exec_spr(io,file,cfg); break;
		case MODE_INFO:    igun_exec_info(io,file,cfg); break;
		case MODE_ACT:     igun_exec_act(io,file,cfg); break;
		case MODE_RSW:     igun_exec_rsw(io,file,cfg); break;
		case MODE_PPJCONV: igun_exec_ppjconv(io,file,cfg); break;
		case MODE_EXT:     igun_exec_ext(io,file,cfg); break;
		}
	}
	if(cfg->mode == MODE_INFO) {
		igun_exec_info_fin(io,cfg);
	}
	io_find_end(find);
}

void igun_exec_gnd(struct io_data* io,const wchar_t *file,struct igun_config *cfg) {
	wchar_t buf[256];
	struct gnd_info *info;
	info = gnd_load(io,file);
	swprintf(buf,_T("%s.bmp"),file);
	gnd_output_bmp(info,io,buf,cfg->gnd_zoom,cfg->gnd_angle,cfg->gnd_grid,cfg->depth);
	gnd_free(info);
}

void igun_exec_gmap(struct io_data* io,const wchar_t *file,struct igun_config *cfg) {
	struct gmap_data* map;
	map = gmap_load(io,file,cfg);
	gmap_output(map,io,file);
	gmap_free(map);
}

void igun_exec_check(struct io_data* io,const wchar_t *file,struct igun_config *cfg) {
	wchar_t *p;
	p = wcsrchr(file,'.');
	if(!p) {
		console_outputwf(_T("igun_exec_check gq܂ : %s\n"),file);
	} else {
		p++;
		if((p[0] == 'R' || p[0] == 'r') && (p[1] == 'S' || p[1] == 's') && (p[2] == 'W' || p[2] == 'w')) {
			rsw_check(io,file,cfg->check_extra);
		} else if((p[0] == 'G' || p[0] == 'g') && (p[1] == 'N' || p[1] == 'n') && (p[2] == 'D' || p[2] == 'd')) {
			gnd_check(io,file,cfg->check_extra);
		} else if((p[0] == 'G' || p[0] == 'g') && (p[1] == 'A' || p[1] == 'a') && (p[2] == 'T' || p[2] == 't')) {
			gat_check(io,file,cfg->check_extra);
		} else if((p[0] == 'A' || p[0] == 'a') && (p[1] == 'C' || p[1] == 'c') && (p[2] == 'T' || p[2] == 't')) {
			act_check(io,file,cfg->check_extra);
		} else {
			console_outputwf(_T("igun_exec_check: unsupported file:%s\n"),file);
		}
	}
}

void igun_exec_spr(struct io_data* io,const wchar_t *file,struct igun_config *cfg) {
	struct spr_data* spr;
	wchar_t buf[256];
	int  j;
	spr = spr_load(io,file);
	if(spr) {
		console_outputf("    outputing bitmaps(%d)\n",spr->frames);
		console_outputf("        ");
		for(j = 0; j < spr->frames ; j++) {
			swprintf(buf,_T("%s_%02d.bmp"),file,j);
			vram_bmpsave(spr->frame[j],io,buf,cfg->depth);
			console_outputf("o");
		}
		console_outputf("\n");
	}
	spr_free(spr);
	console_outputf("    done.\n");
}

void igun_exec_info(struct io_data* io,const wchar_t *file,struct igun_config *cfg) {
	int size_dec;
	if(cfg->info_md5) {
		char buf[256];
		char *p = io_read_all(io,file,&size_dec);
		if(size_dec == -1) {
			strcpy(buf,"--------------------------------");
			size_dec = 0;
		} else {
			MD5_string(p,size_dec,buf);
		}
		free(p);
		console_outputf("%s  ",buf);
	} else {
		size_dec = io_size(io,file);
		if(cfg->info_size) {
			console_outputf("% 8u ",size_dec);
		}
	}
	console_outputwf(_T("%s\n"),file);
	if(0xFFFFFFFF - cfg->info_size_low  < (unsigned)size_dec) {
		cfg->info_size_high++;
		cfg->info_size_low = cfg->info_size_low - (0xFFFFFFFF - size_dec) - 1;
	} else {
		cfg->info_size_low += size_dec;
	}
	cfg->info_file_count++;
}

void igun_exec_info_fin(struct io_data* io,struct igun_config *cfg) {
	if(cfg->info_size || cfg->info_md5) {
		if(cfg->info_size_high) {
			console_outputf(
				"Total %d file(s) , 0x%x%08x byte\n",
				cfg->info_file_count,cfg->info_size_high,cfg->info_size_low
			);
		} else {
			console_outputf("Total %d file(s) , %u byte\n",cfg->info_file_count,cfg->info_size_low);
		}
	}
	cfg->info_file_count = 0;
	cfg->info_size_high  = 0;
	cfg->info_size_low   = 0;
}

void igun_exec_act(struct io_data* io,const wchar_t *file,struct igun_config *cfg) {
	struct act_data * act  = act_load(io,file);
	wchar_t name_w[256];
	char    name_a[256];
	if(act) {
		int i,j;
		RECT rect;
		int size_x,size_y;
		struct vram_data* vram;
		act_size(act,&rect);
		size_x = rect.right  - rect.left + 1;
		size_y = rect.bottom - rect.top  + 1 + 32;
		vram = vram_alloc(size_x,size_y);
		swprintf(name_w,_T("%s.avi"),file);
		vram_aviwrite_start(vram,io,name_w,100,cfg->depth);
		console_outputf("    writing animes (%d)\n",act->animes);
		console_outputf("        ");
		for(i = 0; i < act->animes ; i++) {
			for(j = 0 ; j < act->anime[i].frames; j++) {
				vram_square(vram,0,0,size_x,size_y,0);
				sprintf(name_a,"Anime % 2d Frame % 2d",i,j);
				vram_drawtext(vram,name_a,0,0,vram_RGB(0,0,0));
				act_frame_draw(act,i,j,vram,-rect.left,-rect.top + 32);
				vram_aviwrite_frame(vram);
			}
			console_outputf("o");
		}
		console_outputf("\n    done.\n");
		vram_aviwrite_close(vram);
		vram_free(vram);
	}
	act_free(act);
}

void igun_exec_rsw(struct io_data* io,const wchar_t *file,struct igun_config *cfg) {
	console_outputwf(_T("editing %s\n"),file);
	rsw_edit(file,io,cfg);
}

void igun_exec_ppjconv(struct io_data* io,const wchar_t *file,struct igun_config *cfg) {
	console_outputwf(_T("converting %s\n"),file);
	gmap_conv_platinum(io,file);
}

void igun_exec_ext(struct io_data* io,const wchar_t *file,struct igun_config *cfg) {
	io_copy(io,file,file);
	console_outputwf(_T("extracting %s\n"),file);
}

