
#include "io.h"
#include "io_file.h"

#include <tchar.h>
#include <stdio.h>
#include <stdlib.h>

struct io_file {
	HANDLE hFile;
	int    position;
};

struct io_file* io_file_readopen(const wchar_t *file_) {
	struct io_file *file = calloc(sizeof(struct io_file),1);
	if(file == NULL) {
		return NULL;
	}

	file->hFile = CreateFileW(
		file_,
		GENERIC_READ,
		FILE_SHARE_READ,
		NULL,
		OPEN_EXISTING,
		FILE_ATTRIBUTE_NORMAL,
		NULL
	);
	if(file->hFile == INVALID_HANDLE_VALUE) {
		free(file);
		return NULL;
	}
	file->position = 0;
	return file;
}

int io_file_read(struct io_file *file,void *buf,int len) {
	DWORD temp;
	if(file == NULL) return 0;

	if(ReadFile(file->hFile,buf,len,&temp,NULL) == 0) {
		return 0;
	} else {
		file->position += temp;
		return temp;
	}
}

int io_file_tell(struct io_file *file) {
	return file->position;
}

int io_file_seek(struct io_file *file,int pos) {
	SetFilePointer(file->hFile,pos,0,FILE_BEGIN);
	file->position = pos;
	return 0;
}

int io_file_close(struct io_file *file) {
	CloseHandle(file->hFile);
	free(file);
	return 0;
}

int io_file_size2(struct io_file *file) {
	return GetFileSize(file->hFile,NULL);
}

void* io_file_readall(const wchar_t* file,int *len) {
	DWORD size;
	unsigned int temp;
	void  *buf;

	HANDLE hFile = CreateFileW(
		file,
		GENERIC_READ,
		FILE_SHARE_READ,
		NULL,
		OPEN_EXISTING,
		FILE_ATTRIBUTE_NORMAL | FILE_FLAG_SEQUENTIAL_SCAN,
		NULL
	);
	if(hFile == INVALID_HANDLE_VALUE) {
		if(len) *len = 0;
		return NULL;
	}
	size = GetFileSize(hFile,NULL);
	buf  = malloc(size);
	if(buf == NULL) {
		CloseHandle(hFile);
		if(len) *len = 0;
		return NULL;
	}
	if(ReadFile(hFile,buf,size,&temp,NULL) == 0 || temp != size) {
		// ǂݍݎs
		free(buf);
		CloseHandle(hFile);
		if(len) *len = 0;
		return NULL;
	}
	CloseHandle(hFile);
	if(len) *len = size;
	return buf;
}

int io_file_writeall(const wchar_t* file,void *buf,int len) {
	DWORD  temp;
	HANDLE hFile;

	hFile = CreateFileW(
		file,
		GENERIC_WRITE,
		0,
		NULL,
		CREATE_ALWAYS,
		FILE_ATTRIBUTE_NORMAL | FILE_FLAG_SEQUENTIAL_SCAN,
		NULL
	);
	if(hFile == INVALID_HANDLE_VALUE) {
		// fBNg쐬Ă݂
		wchar_t dir[1024];
		int i = 0;
		wcscpy(dir,file);
		while(dir[i]) {
			if(dir[i] == '\\' || dir[i] == '/') {
				wchar_t c = dir[i];
				dir[i] = 0;
				CreateDirectory(dir,NULL);
				dir[i] = c;
			}
			i++;
		}
		hFile = CreateFileW(
			file,
			GENERIC_WRITE,
			0,
			NULL,
			CREATE_ALWAYS,
			FILE_ATTRIBUTE_NORMAL | FILE_FLAG_SEQUENTIAL_SCAN,
			NULL
		);
		if(hFile == INVALID_HANDLE_VALUE) {
			return 0;
		}
	}
	if(WriteFile(hFile,buf,len,&temp,NULL) == 0 || temp != len) {
		// ݎs
		CloseHandle(hFile);
		return 0;
	}
	CloseHandle(hFile);
	return 1;
}

int io_file_size(const wchar_t* file) {
	int size;
	HANDLE hFile = CreateFileW(
		file,
		GENERIC_READ,
		FILE_SHARE_READ,
		NULL,
		OPEN_EXISTING,
		FILE_ATTRIBUTE_NORMAL | FILE_FLAG_SEQUENTIAL_SCAN,
		NULL
	);
	if(hFile == INVALID_HANDLE_VALUE) {
		return 0;
	}
	size = GetFileSize(hFile,NULL);
	CloseHandle(hFile);
	return size;
}

const wchar_t *io_file_find(struct io_find *find) {
	const wchar_t* ret = NULL;
	if(!find->find_handle) {
		find->find_handle = FindFirstFileW(_T("*"),&find->find_data);
		if(find->find_handle == INVALID_HANDLE_VALUE) {
			ret = NULL;
		} else {
			ret = find->find_data.cFileName;
		}
	} else {
		if(!FindNextFileW(find->find_handle,&find->find_data)) {
			FindClose(find->find_handle);
			ret = NULL;
		} else {
			ret = find->find_data.cFileName;
		}
	}
	if(ret == NULL) {
		return NULL;
	}
	if(!wcscmp(_T("."),ret) || !wcscmp(_T(".."),ret)) {
		// . .. O
		return io_file_find(find);
	} else if(GetFileAttributesW(ret) & FILE_ATTRIBUTE_DIRECTORY) {
		// fBNg
		return io_file_find(find);
	}
	return ret;
}
