
#include <tchar.h>
#include <stdio.h>
#include <windows.h>
#include "rsw.h"
#include "rsw_edit.h"
#include "gmap.h"
#include "io/io.h"
#include "resource.h"
#include "console.h"

struct rsw_data *rsw_load(struct io_data *io,const wchar_t* file) {
	struct rsw_data *rsw;
	char *fp, *fp_base;
	int i,size;
	rsw = (struct rsw_data*)calloc(1,sizeof(struct rsw_data));
	if(rsw == NULL) { return NULL; }
	console_outputwf(_T("    reading %s\n"),file);
	fp = fp_base = (char*)io_read_all(io,file,&size);

	memcpy(&rsw->head,fp,166); fp += 166;
	memcpy(&rsw->info,fp, 80); fp +=  80;
	rsw->obj_max   = rsw->info.ModelCount + 16;
	rsw->obj       = (struct rsw_object*)calloc(sizeof(struct rsw_object),rsw->obj_max);
	if(rsw->head.magic == 0x0102) {
		rsw->obj_len[1] = 248;	rsw->obj_len[2] = 108;
		rsw->obj_len[3] = 192;	rsw->obj_len[4] = 116;
	} else {
		rsw->obj_len[1] = 248;	rsw->obj_len[2] = 108;
		rsw->obj_len[3] = 188;	rsw->obj_len[4] = 0;
	}
	for(i = 0;i < rsw->info.ModelCount ; i++) {
		if(fp_base + size <= fp + sizeof(int)) {
			rsw->obj[i].type = -1;
		} else {
			rsw->obj[i].type = *(int*)(fp); fp += sizeof(int);
		}
		if(rsw->obj[i].type < 1 || rsw->obj[i].type > 4) {
			console_outputf(
				"        warning : unsupported type %d. Resize model count to %d\n",
				rsw->obj[i].type,i
			);
			rsw->info.ModelCount = i;
		} else {
			int len = rsw->obj_len[rsw->obj[i].type];
			rsw->obj[i].buf = malloc(len);
			if(rsw->obj[i].buf == NULL) {
				rsw_free(rsw); free(fp_base); return NULL;
			}
            memcpy(rsw->obj[i].buf,fp,len); fp += len;
			rsw->obj[i].len = len;
		}
	}
	rsw->extra_len = size - (fp - fp_base);
	if(rsw->extra_len > 0) {
		rsw->extra_data = malloc(rsw->extra_len);
		memcpy(rsw->extra_data,fp,rsw->extra_len); // fp += rsw->extra_len;
	}
	console_outputf("    extra data %d byte\n",rsw->extra_len);
	free(fp_base);
	return rsw;
}

void rsw_save(struct rsw_data* rsw,struct io_data *io,const wchar_t *file) {
	int  i;
	struct io_fp *fp;

	console_outputwf(_T("    writing %s\n"),file);
	fp = io_write_open(io,file);
	if(fp == NULL) { return; }
	io_write(fp,&rsw->head,166);
	io_write(fp,&rsw->info,80);
	for(i = 0; i < rsw->info.ModelCount ; i++) {
		io_write(fp,&rsw->obj[i].type,sizeof(int));
		io_write(fp,rsw->obj[i].buf,rsw->obj[i].len);
	}
	if(rsw->extra_len) {
		io_write(fp,rsw->extra_data,rsw->extra_len);
	}
	io_close(fp);
}

void rsw_free(struct rsw_data *rsw) {
	int i;
	if(rsw == NULL) { return; }
	if(rsw->obj) {
		for(i = 0 ; i < rsw->info.ModelCount ; i++) {
			free(rsw->obj[i].buf);
		}
		free(rsw->obj);
	}
	free(rsw->extra_data);
	free(rsw);
}

void rsw_output_map(struct gmap_data* map,struct io_data* io,wchar_t *file_pre) {
	wchar_t  *p;
	short temp_s;
	long  temp_l;
	char  blank[256];
	wchar_t  file[256];
	struct io_fp *fp;
	if(map == NULL) { return; }
	swprintf(file,_T("%s.rsw"),file_pre);
	console_outputwf(_T("    writing %s\n"),file);

	fp = io_write_open(io,file);
	if(fp == NULL) { return; }

	// file_pre  \\ ܂܂Ă炻܂ō
	p = wcsrchr(file_pre,'/');
	if(p != NULL && p[1] != 0) { file_pre = p + 1; }
	p = wcsrchr(file_pre,'\\');
	if(p != NULL && p[1] != 0) { file_pre = p + 1; }

	memset(blank,0,256);
	io_write(fp,"GRSW",4);
	temp_s = 0x0901; // 0x0102;
	io_write(fp,&temp_s,sizeof(short));
	io_write(fp,blank,40);

	// gnd file
	temp_l = io_printfWA(fp,_T("%s.gnd"),file_pre);
	io_write(fp,blank,40 - temp_l);

	// gat file
	temp_l = io_printfWA(fp,_T("%s.gat"),file_pre);
	io_write(fp,blank,40 - temp_l);

	// dummy
	io_write(fp,blank,42);

	// ʂ̍H
	temp_l = 0x41B8;
	io_write(fp,&temp_l,sizeof(int));

	// dummy
	io_write(fp,blank,10);

	// ̌ŒoCg
	temp_l = 0x42480000;
	io_write(fp,&temp_l,sizeof(int));

	temp_l = 0x03;
	io_write(fp,&temp_l,sizeof(int));
	temp_l = 0x2D;
	io_write(fp,&temp_l,sizeof(int));
	temp_l = 0x2D;
	io_write(fp,&temp_l,sizeof(int));

	// eNX`[̃J[}XN
	// mask-R
	temp_l = 0x3F800000;
	io_write(fp,&temp_l,sizeof(int));

	// mask-G
	temp_l = 0x3F800000;
	io_write(fp,&temp_l,sizeof(int));

	// mask-B
	temp_l = 0x3F800000;
	io_write(fp,&temp_l,sizeof(int));

	temp_l = 0x3F0CCCCD;
	io_write(fp,&temp_l,sizeof(int));
	temp_l = 0x3F000000;
	io_write(fp,&temp_l,sizeof(int));
	temp_l = 0x3F000000;
	io_write(fp,&temp_l,sizeof(int));
	temp_l = 0x3F000000;
	io_write(fp,&temp_l,sizeof(int));

	temp_l = 0x09;
	io_write(fp,&temp_l,sizeof(int));
	temp_l = 0x84D4;
	io_write(fp,&temp_l,sizeof(int));
	temp_l = 0x0200;
	io_write(fp,&temp_l,sizeof(int));
	temp_l = 0x0000;
	io_write(fp,&temp_l,sizeof(int));

	// objs : 0
	temp_l = 0;
	io_write(fp,&temp_l,sizeof(int));

	io_close(fp);
}

// rsw `FbN[`B
// tH[}bg͗pɓKɍ㕨Ȃ̂ŁAڂłB

void rsw_check(struct io_data* io,const wchar_t *file,int extra_info) {
	char *fp;
	char *fp_base;
	int len[16];
	int count = 0;
	int magic;
	int filesize;
	int objs;
	int i;
	unsigned char buf[256];
	fp = (char*)io_read_all(io,file,&filesize);
	fp_base = fp;
	console_outputwf(_T("Checking %s ...\n"),file);
	if(fp == NULL) {
		console_outputwf(_T("    File %s can't open.\n"),file);
		return;
	}
	memcpy(buf,fp,6); fp += 6;
	magic = *(unsigned short *)(buf + 4);
	buf[4] = 0;

	fp += 0x00EC;
	memcpy(&objs,fp,sizeof(int)); fp += sizeof(int);
	console_outputf("    Code:%s Magic:%02x Objs:%d\n",buf,magic,objs);
	if(magic == 0x0102) {
		len[0] = 0; len[1] = 248; len[2] = 108; len[3] = 192; len[4] = 116;
	} else {
		len[0] = 0; len[1] = 248; len[2] = 108; len[3] = 188; len[4] = 0;
	}
	for(i = 0;i < objs ; i++) {
		int type;
		memcpy(&type,fp,4); fp += 4;
		if(extra_info) {
			console_outputf("    Obj[%d] type:%d pos:0x%08x",++count,type,(fp - fp_base)-4);
		}
		if(type == 1) {
			memcpy(buf,fp,len[1]); fp += len[1];
			if(extra_info) {
				console_outputf(" rsw : %s",&buf[52]);
			}
		} else if(type == 2 && len[2]) {
			memcpy(buf,fp,len[2]); fp += len[2];
		} else if(type == 3) {
			memcpy(buf,fp,len[3]); fp += len[3];
			if(extra_info) {
				console_outputf(" wav : %s",&buf[80]);
			}
		} else if(type == 4 && len[4]) {
			memcpy(buf,fp,len[4]); fp += len[4];
		} else {
			if(extra_info) {
				console_outputf("    ERR: Unknown type %08x pos 0x%x\n",type,(fp - fp_base) - 4);
			}
			free(fp_base);
			return;
		}
		if(extra_info) {
			console_outputf("\n");
		}
	}
	if(filesize != (fp - fp_base)) {
		if(magic == 0x0102 && filesize - (fp - fp_base) == 65520) {
			// ̂magic 0x0102 65520 oCg̃f[^tĂBB
			console_outputf("    Read Ok 0x%x\n",(fp - fp_base) + 65520);
		} else {
			console_outputf("    ERR: Left %d Byte\n",filesize - (fp - fp_base));
		}
	} else {
		console_outputf("    Read Ok 0x%x\n",(fp - fp_base));
	}
	free(fp_base);
}

