
#include <tchar.h>
#include <stdio.h>
#include <windows.h>

#include "igun.h"
#include "io/io.h"
#include "rsw.h"
#include "rsw_edit.h"
#include "resource.h"

struct rsw_data    *rsw_edit_data;
struct igun_config *rsw_edit_cfg;
struct io_data     *rsw_edit_io;

void rsw_edit_obj_move(HWND hWnd);

BOOL CALLBACK rsw_edit_proc(HWND hWnd,UINT uMsg,WPARAM wParam,LPARAM lParam);

void rsw_edit(const wchar_t *file,struct io_data *io,struct igun_config *cfg) {
	struct rsw_data *rsw = rsw_load(io,file);
	int ret;
	int resource = (cfg->locate == LANG_JAPANESE ? RSW_EDIT_JP : RSW_EDIT_EN);
	if(rsw == NULL) { return; }
	rsw_edit_io   = io;
	rsw_edit_cfg  = cfg;
	rsw_edit_data = rsw;

	ret = DialogBoxW(NULL,MAKEINTRESOURCE(resource),NULL,rsw_edit_proc);
	if(ret == 0) {
		// Z[u
		rsw_save(rsw,io,file);
	}
	rsw_edit_io   = NULL;
	rsw_edit_cfg  = NULL;
	rsw_edit_data = NULL;
	rsw_free(rsw);
}

void rsw_edit_setfloat(HWND hWnd,int item,float data) {
	wchar_t buf[64];
	swprintf(buf,_T("%.2f"),data);
	SetDlgItemText(hWnd,item,buf);
}

void rsw_edit_getfloat(HWND hWnd,int item,float *data) {
	wchar_t buf[256];
	GetDlgItemText(hWnd,item,buf,256);
	*data = (float)_wtof(buf);
}

void rsw_edit_obj_add(HWND hWnd,struct rsw_data* rsw,int index) {
	wchar_t buf[256];
	wchar_t name_w[256];
	struct rsw_object *obj;
	if(rsw == NULL) {
		wcscpy(buf,_T("(null)"));
	} else {
		obj = &rsw->obj[index];
		if(obj->type == 1) {
			swprintf(buf,_T("%04x rsw : %s"),obj->type,io_mbcs_to_unicode(rsw_edit_io,name_w,&obj->buf[52]));
		} else if(obj->type == 2) {
			swprintf(buf,_T("%04x : unknown"),obj->type);
		} else if(obj->type == 3) {
			swprintf(buf,_T("%04x : wav %s"),obj->type,io_mbcs_to_unicode(rsw_edit_io,name_w,&obj->buf[80]));
		} else if(obj->type == 4) {
			swprintf(buf,_T("%04x : unknown"),obj->type);
		}
	}
	SendDlgItemMessage(hWnd,RSW_OBJECT,LB_INSERTSTRING,index,(LPARAM)(LPCTSTR)buf);
}

void rsw_edit_obj_load(HWND hWnd,struct rsw_data* rsw,int pos) {
	int readonly;
	struct rsw_object_t01 *obj;
	if(pos == LB_ERR || rsw->obj[pos].type != 1) {
		// Type01ȊO
		readonly = 1;
	} else {
		wchar_t name_w[256];

		obj = (struct rsw_object_t01*)rsw->obj[pos].buf;
		readonly = 0;
		rsw_edit_setfloat(hWnd,RSW_T01_PX,obj->pos.pos_x);
		rsw_edit_setfloat(hWnd,RSW_T01_PY,obj->pos.pos_y);
		rsw_edit_setfloat(hWnd,RSW_T01_PZ,obj->pos.pos_z);
		rsw_edit_setfloat(hWnd,RSW_T01_RX,obj->pos.rot_x);
		rsw_edit_setfloat(hWnd,RSW_T01_RY,obj->pos.rot_y);
		rsw_edit_setfloat(hWnd,RSW_T01_RZ,obj->pos.rot_z);
		rsw_edit_setfloat(hWnd,RSW_T01_SX,obj->pos.size_x);
		rsw_edit_setfloat(hWnd,RSW_T01_SY,obj->pos.size_y);
		rsw_edit_setfloat(hWnd,RSW_T01_SZ,obj->pos.size_z);

		io_mbcs_to_unicode(rsw_edit_io,name_w,obj->LayerName);
		SetDlgItemText(hWnd,RSW_T01_LAYER,name_w);
		io_mbcs_to_unicode(rsw_edit_io,name_w,obj->RSMName);
		SetDlgItemText(hWnd,RSW_T01_RSM,name_w);
		io_mbcs_to_unicode(rsw_edit_io,name_w,obj->ModelType);
		SetDlgItemText(hWnd,RSW_T01_MODEL,name_w);
	}
	SendDlgItemMessage(hWnd,RSW_T01_PX,EM_SETREADONLY,readonly,0);
	SendDlgItemMessage(hWnd,RSW_T01_PY,EM_SETREADONLY,readonly,0);
	SendDlgItemMessage(hWnd,RSW_T01_PZ,EM_SETREADONLY,readonly,0);
	SendDlgItemMessage(hWnd,RSW_T01_RX,EM_SETREADONLY,readonly,0);
	SendDlgItemMessage(hWnd,RSW_T01_RY,EM_SETREADONLY,readonly,0);
	SendDlgItemMessage(hWnd,RSW_T01_RZ,EM_SETREADONLY,readonly,0);
	SendDlgItemMessage(hWnd,RSW_T01_SX,EM_SETREADONLY,readonly,0);
	SendDlgItemMessage(hWnd,RSW_T01_SY,EM_SETREADONLY,readonly,0);
	SendDlgItemMessage(hWnd,RSW_T01_SZ,EM_SETREADONLY,readonly,0);
	SendDlgItemMessage(hWnd,RSW_T01_LAYER,EM_SETREADONLY,readonly,0);
	SendDlgItemMessage(hWnd,RSW_T01_RSM,EM_SETREADONLY,readonly,0);
	SendDlgItemMessage(hWnd,RSW_T01_MODEL,EM_SETREADONLY,readonly,0);
}

void rsw_edit_obj_save(HWND hWnd,struct rsw_data* rsw,int pos) {
	struct rsw_object_t01 *obj;
	if(pos != LB_ERR && rsw->obj[pos].type == 1) {
		wchar_t name_w[256];
		obj = (struct rsw_object_t01*)rsw->obj[pos].buf;
		rsw_edit_getfloat(hWnd,RSW_T01_PX,&obj->pos.pos_x);
		rsw_edit_getfloat(hWnd,RSW_T01_PY,&obj->pos.pos_y);
		rsw_edit_getfloat(hWnd,RSW_T01_PZ,&obj->pos.pos_z);
		rsw_edit_getfloat(hWnd,RSW_T01_RX,&obj->pos.rot_x);
		rsw_edit_getfloat(hWnd,RSW_T01_RY,&obj->pos.rot_y);
		rsw_edit_getfloat(hWnd,RSW_T01_RZ,&obj->pos.rot_z);
		rsw_edit_getfloat(hWnd,RSW_T01_SX,&obj->pos.size_x);
		rsw_edit_getfloat(hWnd,RSW_T01_SY,&obj->pos.size_y);
		rsw_edit_getfloat(hWnd,RSW_T01_SZ,&obj->pos.size_z);

		GetDlgItemText(hWnd,RSW_T01_LAYER,name_w,256);
		io_unicode_to_mbcs(rsw_edit_io,obj->ModelType,name_w);

		GetDlgItemText(hWnd,RSW_T01_RSM,name_w,256);
		io_unicode_to_mbcs(rsw_edit_io,obj->RSMName,name_w);

		GetDlgItemText(hWnd,RSW_T01_MODEL,name_w,256);
		io_unicode_to_mbcs(rsw_edit_io,obj->ModelType,name_w);
	}
}

char* rsw_edit_obj_decode(char *buf,int *len) {
	int srclen = strlen(buf);
	int dstlen = 0;
	unsigned char *dest = malloc(srclen / 2 + 16);
	int flag   = 0;
	int i,c;
	for(i = 0; i < srclen;i++) {
		if(*buf >= '0' && *buf <= '9') {
			c = *buf - '0';
		} else if(*buf >= 'a' && *buf <= 'f') {
			c = *buf - 'a' + 10;
		} else if(*buf >= 'A' && *buf <= 'F') {
			c = *buf - 'A' + 10;
		} else {
			c = -1;
		}
		if(c >= 0) {
			if(flag) {
				dest[dstlen] = dest[dstlen] * 16 + c;
				dstlen++; flag = 0;
			} else {
				dest[dstlen] = c;
				flag = 1;
			}
		}
		buf++;
	}
	*len = dstlen;
	return (char*)dest;
}

BOOL CALLBACK rsw_edit_proc(HWND hWnd,UINT uMsg,WPARAM wParam,LPARAM lParam) {
	static pos = LB_ERR;
	int i;
	struct rsw_data* rsw = rsw_edit_data;
	switch( uMsg ) {
	case WM_INITDIALOG:
		rsw_edit_setfloat(hWnd,RSW_HEAD_WATER_HEIGHT,rsw->info.WaterHeight);
		rsw_edit_setfloat(hWnd,RSW_HEAD_WATER1,rsw->info.WaterWave_Power);
		rsw_edit_setfloat(hWnd,RSW_HEAD_WATER2,rsw->info.WaterWave_Speed);
		rsw_edit_setfloat(hWnd,RSW_HEAD_WATER3,rsw->info.WaterWave_Height);
		rsw_edit_setfloat(hWnd,RSW_HEAD_LIGHT_R1,rsw->info.LightDiffuse_R);
		rsw_edit_setfloat(hWnd,RSW_HEAD_LIGHT_G1,rsw->info.LightDiffuse_G);
		rsw_edit_setfloat(hWnd,RSW_HEAD_LIGHT_B1,rsw->info.LightDiffuse_B);
		rsw_edit_setfloat(hWnd,RSW_HEAD_LIGHT_R2,rsw->info.LightAmbient_R);
		rsw_edit_setfloat(hWnd,RSW_HEAD_LIGHT_G2,rsw->info.LightAmbient_G);
		rsw_edit_setfloat(hWnd,RSW_HEAD_LIGHT_B2,rsw->info.LightAmbient_B);

		// ʐݒ̒ǉ
		SendDlgItemMessage(hWnd,RSW_HEAD_WATER_COLOR,CB_ADDSTRING,0,(LPARAM)_T("00 : Normal light colored water (Prontera)"));
		SendDlgItemMessage(hWnd,RSW_HEAD_WATER_COLOR,CB_ADDSTRING,0,(LPARAM)_T("01 : A little darker water"));
		SendDlgItemMessage(hWnd,RSW_HEAD_WATER_COLOR,CB_ADDSTRING,0,(LPARAM)_T("02 : Very light/tropical water (Jawaii)"));
		SendDlgItemMessage(hWnd,RSW_HEAD_WATER_COLOR,CB_ADDSTRING,0,(LPARAM)_T("03 : Very dark water (Culvert)"));
		SendDlgItemMessage(hWnd,RSW_HEAD_WATER_COLOR,CB_ADDSTRING,0,(LPARAM)_T("04 : Lava (Magma Dungeon)"));
		SendDlgItemMessage(hWnd,RSW_HEAD_WATER_COLOR,CB_ADDSTRING,0,(LPARAM)_T("05 : Dark/muddy water"));
		SendDlgItemMessage(hWnd,RSW_HEAD_WATER_COLOR,CB_SETCURSEL,rsw->info.WaterType,0);
		// IuWFNgꗗ̕\
		for(i = 0;i < rsw->info.ModelCount ; i++) {
			rsw_edit_obj_add(hWnd,rsw,i);
		}
		pos = LB_ERR;
		break;
	case WM_CLOSE:
	case WM_COMMAND:
		if(uMsg == WM_CLOSE || (uMsg == WM_COMMAND && LOWORD(wParam) == IDOK)) {
			// IuWFNg̏㏑
			rsw_edit_obj_save(hWnd,rsw,pos);
			// rsw wb_̏㏑
			rsw_edit_getfloat(hWnd,RSW_HEAD_WATER_HEIGHT,&rsw->info.WaterHeight);
			rsw_edit_getfloat(hWnd,RSW_HEAD_WATER1,&rsw->info.WaterWave_Power);
			rsw_edit_getfloat(hWnd,RSW_HEAD_WATER2,&rsw->info.WaterWave_Speed);
			rsw_edit_getfloat(hWnd,RSW_HEAD_WATER3,&rsw->info.WaterWave_Height);
			rsw_edit_getfloat(hWnd,RSW_HEAD_LIGHT_R1,&rsw->info.LightDiffuse_R);
			rsw_edit_getfloat(hWnd,RSW_HEAD_LIGHT_G1,&rsw->info.LightDiffuse_G);
			rsw_edit_getfloat(hWnd,RSW_HEAD_LIGHT_B1,&rsw->info.LightDiffuse_B);
			// rsw_edit_getfloat(hWnd,RSW_HEAD_LIGHT_R2,&rsw->info.LightAmbient_R);
			// rsw_edit_getfloat(hWnd,RSW_HEAD_LIGHT_G2,&rsw->info.LightAmbient_G);
			// rsw_edit_getfloat(hWnd,RSW_HEAD_LIGHT_B2,&rsw->info.LightAmbient_B);
			rsw->info.WaterType =
				SendDlgItemMessage(hWnd,RSW_HEAD_WATER_COLOR,CB_GETCURSEL,0,0);
			if(rsw->info.WaterType == CB_ERR) {
				rsw->info.WaterType = 0;
			}
			EndDialog(hWnd,0);
		} else if(uMsg == WM_COMMAND) {
			switch(LOWORD(wParam)) {
			case IDCANCEL:
				// Z[uȂ
				EndDialog(hWnd,1);
				break;
			case RSW_OBJECT:
				if(HIWORD(wParam) == LBN_SELCHANGE) {
					// IuWFNg̏㏑
					rsw_edit_obj_save(hWnd,rsw,pos);
					pos = SendDlgItemMessage(hWnd,RSW_OBJECT,LB_GETCURSEL,0,0);
					rsw_edit_obj_load(hWnd,rsw,pos);
				}
				break;
			case RSW_OBJECT_NEW:
				MessageBox(hWnd,_T("sɂB"),_T("rsw_edit.c"),MB_OK);
				break;
			case RSW_OBJECT_UP:
				if(pos > 0) {
					struct rsw_object obj = rsw->obj[pos];
					// IuWFNg̏㏑
					rsw_edit_obj_save(hWnd,rsw,pos);
					// object ւ
					rsw->obj[pos] = rsw->obj[pos-1];
					rsw->obj[pos-1] = obj;
					// listboxւ
					SendDlgItemMessage(hWnd,RSW_OBJECT,LB_DELETESTRING,pos-1,0);
					rsw_edit_obj_add(hWnd,rsw,pos);
					SendDlgItemMessage(hWnd,RSW_OBJECT,LB_SETCURSEL,pos-1,0);
					pos--;
				} else {
					MessageBox(hWnd,_T("ړł܂B"),_T("debug"),MB_OK);
				}
				break;
			case RSW_OBJECT_DOWN:
				if(pos >= 0 && pos < rsw->info.ModelCount - 1) {
					struct rsw_object obj = rsw->obj[pos];
					// IuWFNg̏㏑
					rsw_edit_obj_save(hWnd,rsw,pos);
					// object ւ
					rsw->obj[pos] = rsw->obj[pos+1];
					rsw->obj[pos+1] = obj;
					// listboxւ
					SendDlgItemMessage(hWnd,RSW_OBJECT,LB_DELETESTRING,pos,0);
					rsw_edit_obj_add(hWnd,rsw,pos+1);
					SendDlgItemMessage(hWnd,RSW_OBJECT,LB_SETCURSEL,pos+1,0);
					pos++;
				} else {
					MessageBox(hWnd,_T("ړł܂B"),_T("debug"),MB_OK);
				}
				break;
			case RSW_OBJECT_DEL:
				if(pos != LB_ERR) {
					// IuWFNg{̂
					free(rsw->obj[pos].buf);
					memmove(
						&rsw->obj[pos],&rsw->obj[pos+1],
						(rsw->info.ModelCount - pos - 1) * sizeof(struct rsw_object)
					);
					rsw->info.ModelCount--;
					rsw->obj[rsw->info.ModelCount].buf = NULL;
					// listbox̓e
					SendDlgItemMessage(hWnd,RSW_OBJECT,LB_DELETESTRING,pos,0);
					// Iꏊ̕ύX
					if(rsw->info.ModelCount == 0) {
						pos = -1;
					} else if(pos == rsw->info.ModelCount) {
						pos = rsw->info.ModelCount - 1;
					} else {
						;
					}
					SendDlgItemMessage(hWnd,RSW_OBJECT,LB_SETCURSEL,pos,0);
					if(pos == -1) {
						pos = LB_ERR;
					} else {
						rsw_edit_obj_load(hWnd,rsw,pos);
					}
				}
				break;
			case RSW_OBJECT_DUPE:
				if(pos != LB_ERR) {
					if(rsw->info.ModelCount == rsw->obj_max) {
						rsw->obj_max += 32;
						rsw->obj = realloc(rsw->obj,rsw->obj_max * sizeof(struct rsw_object));
					}
					// ݂̐ݒۑ
					rsw_edit_obj_save(hWnd,rsw,pos);
					// ̈ړ
					memmove(
						&rsw->obj[pos + 1],&rsw->obj[pos],
						(rsw->info.ModelCount - pos) * sizeof(struct rsw_object)
					);
					// obt@Rs[
					rsw->obj[pos+1].buf = malloc(rsw->obj[pos+1].len);
					memcpy(rsw->obj[pos+1].buf,rsw->obj[pos].buf,rsw->obj[pos+1].len);
					rsw->info.ModelCount++;
					// Iꏊɕ}
					rsw_edit_obj_add(hWnd,rsw,pos);
				}
				break;
			case RSW_OBJECT_MOVE:
				// ݂̐ݒۑ
				rsw_edit_obj_save(hWnd,rsw,pos);
				// ړāA
				rsw_edit_obj_move(hWnd);
				// ݂̐ݒǂݍ
				rsw_edit_obj_load(hWnd,rsw,pos);
				break;
			case RSW_OBJECT_COPY:
				// Ȃ񂾂悱̉\[Xi
				if(pos != LB_ERR) {
					char *buf = (char*)GlobalAlloc(GMEM_FIXED,rsw->obj[pos].len * 4 + 16);
					char *p   = buf;
					unsigned char *p2 = (unsigned char*)rsw->obj[pos].buf;
					int i;
					// ݂̐ݒۑ
					rsw_edit_obj_save(hWnd,rsw,pos);
					// ϊ
					i = rsw->obj[pos].type;
					p += sprintf(p,"%02x ", i & 0x000000FF);
					p += sprintf(p,"%02x ",(i & 0x0000FF00) >>  8);
					p += sprintf(p,"%02x ",(i & 0x00FF0000) >> 16);
					p += sprintf(p,"%02x ",(i & 0xFF000000) >> 24);
					for(i = 0 ; i < rsw->obj[pos].len ; i++) {
						p += sprintf(p,"%02x",p2[i]);
						if((i + 5) % 16 == 0) {
							p += sprintf(p,"\r\n");
						} else {
							*(p++) = ' ';
						}
					}
					*p = 0;
					if(OpenClipboard(hWnd) == 0) {
						MessageBox(hWnd,_T("Nbv{[hgpłB"),_T("rsw.c"),MB_OK);
						GlobalFree(buf);
					} else {
						EmptyClipboard();
						SetClipboardData(CF_TEXT,buf);
						CloseClipboard();
						// mۂȂ̂̓}i[炵
						// GlobalFree(buf);
					}
				}
				break;
			case RSW_OBJECT_PASTE:
				// ̕Ɖorz
				if(OpenClipboard(hWnd) == 0) {
					MessageBox(hWnd,_T("Nbv{[hgpłB"),_T("rsw.c"),MB_OK);
				} else {
					char *buf = GetClipboardData(CF_TEXT);
					if(buf == NULL || buf[0] == 0) {
						MessageBox(hWnd,_T("Nbv{[h̃f[^ǂݎ܂B"),_T("rsw.c"),MB_OK);
					} else {
						int  len;
						char *buf2 = rsw_edit_obj_decode(buf,&len);
						int  type  = *(int *)buf2;
						int  paste_pos;
						// `FbN
						if(type < 0 || type > 4 || rsw->obj_len[type] + 4 != len) {
							MessageBox(hWnd,_T("Nbv{[h̃f[^słB"),_T("rsw.c"),MB_OK);
						} else {
							if(rsw->info.ModelCount == rsw->obj_max) {
								rsw->obj_max += 32;
								rsw->obj = realloc(rsw->obj,rsw->obj_max * sizeof(struct rsw_object));
							}
							// \tꏊmۂ
							if(pos == LB_ERR) {
								paste_pos = rsw->info.ModelCount;
							} else {
								paste_pos = pos + 1;
							}
							// ̈ړRs[
							memmove(
								&rsw->obj[paste_pos + 1],&rsw->obj[paste_pos],
								(rsw->info.ModelCount - paste_pos) * sizeof(struct rsw_object)
							);
							rsw->obj[paste_pos].type = type;
							rsw->obj[paste_pos].len  = len - 4;
							rsw->obj[paste_pos].buf  = (char *)malloc(len);
							memcpy(rsw->obj[paste_pos].buf,buf2 + 4,len - 4);
							rsw->info.ModelCount++;
							// IuWFNgꗗ̍XV
							rsw_edit_obj_add(hWnd,rsw,paste_pos);
						}
						free(buf2);
					}
					CloseClipboard();
				}
				break;
			}
		}
	}
	return 0;
}

struct {
	float mul_x;
	float mul_y;
	float mul_z;
	float add_x;
	float add_y;
	float add_z;
} rsw_obj_move_data;

BOOL CALLBACK rsw_edit_obj_move_proc(HWND hWnd,UINT uMsg,WPARAM wParam,LPARAM lParam);

void rsw_edit_obj_move(HWND hWnd) {
	int resource = (rsw_edit_cfg->locate == LANG_JAPANESE ? RSW_EDIT_MOVE_JP : RSW_EDIT_MOVE_EN);
	rsw_obj_move_data.mul_x = 1;
	rsw_obj_move_data.mul_y = 1;
	rsw_obj_move_data.mul_z = 1;
	rsw_obj_move_data.add_x = 0;
	rsw_obj_move_data.add_y = 0;
	rsw_obj_move_data.add_z = 0;
	if(DialogBoxW(NULL,MAKEINTRESOURCE(resource),hWnd,rsw_edit_obj_move_proc) == 0) {
		int i;
		struct rsw_object_t01 *obj;
		for(i = 0; i < rsw_edit_data->info.ModelCount ; i++) {
			if(rsw_edit_data->obj[i].type == 1) {
				obj = (struct rsw_object_t01*)rsw_edit_data->obj[i].buf;
				obj->pos.pos_x = obj->pos.pos_x * rsw_obj_move_data.mul_x + rsw_obj_move_data.add_x;
				obj->pos.pos_y = obj->pos.pos_y * rsw_obj_move_data.mul_y + rsw_obj_move_data.add_y;
				obj->pos.pos_z = obj->pos.pos_z * rsw_obj_move_data.mul_z + rsw_obj_move_data.add_z;
			}
		}
	}
}

BOOL CALLBACK rsw_edit_obj_move_proc(HWND hWnd,UINT uMsg,WPARAM wParam,LPARAM lParam) {
	switch( uMsg ) {
	case WM_INITDIALOG:
		rsw_edit_setfloat(hWnd,RSW_MUL_X,rsw_obj_move_data.mul_x);
		rsw_edit_setfloat(hWnd,RSW_MUL_Y,rsw_obj_move_data.mul_y);
		rsw_edit_setfloat(hWnd,RSW_MUL_Z,rsw_obj_move_data.mul_z);
		rsw_edit_setfloat(hWnd,RSW_ADD_X,rsw_obj_move_data.add_x);
		rsw_edit_setfloat(hWnd,RSW_ADD_Y,rsw_obj_move_data.add_y);
		rsw_edit_setfloat(hWnd,RSW_ADD_Z,rsw_obj_move_data.add_z);
		break;
	case WM_CLOSE:
	case WM_COMMAND:
		if(uMsg == WM_CLOSE || (uMsg == WM_COMMAND && LOWORD(wParam) == IDOK)) {
			rsw_edit_getfloat(hWnd,RSW_MUL_X,&rsw_obj_move_data.mul_x);
			rsw_edit_getfloat(hWnd,RSW_MUL_Y,&rsw_obj_move_data.mul_y);
			rsw_edit_getfloat(hWnd,RSW_MUL_Z,&rsw_obj_move_data.mul_z);
			rsw_edit_getfloat(hWnd,RSW_ADD_X,&rsw_obj_move_data.add_x);
			rsw_edit_getfloat(hWnd,RSW_ADD_Y,&rsw_obj_move_data.add_y);
			rsw_edit_getfloat(hWnd,RSW_ADD_Z,&rsw_obj_move_data.add_z);
			EndDialog(hWnd,0);
		} else if(uMsg == WM_COMMAND && LOWORD(wParam) == IDCANCEL) {
			// ړȂ
			EndDialog(hWnd,1);
		}
		break;
	}
	return 0;
}

