
#include <tchar.h>
#include <stdio.h>

#include "spr.h"
#include "vram.h"
#include "io/io.h"
#include "console.h"

struct spr_data* spr_load(struct io_data* io,const wchar_t *file) {
	struct spr_data* spr;
	unsigned char *buf,*p;
	unsigned char **frame_data;
	struct vram_size *frame_size;
	int size;
	int i;

	spr = calloc(sizeof(struct spr_data),1);
	if(spr == NULL) { return NULL; }

	console_outputwf(_T("    reading %s ...\n"),file);
	buf = io_read_all(io,file,&size);
	if(buf == NULL) {
		console_outputwf(_T("        File %s can't open.\n"),file);
		free(spr);
		return NULL;
	}
	if(buf[0] == 'B' && buf[1] == 'M') {
		// BMP Ȃ̂spr `Ƀbvďo͂
		spr->frames = 1;
		spr->frame = (struct vram_data**)malloc(sizeof(struct vram_data*));
		spr->frame[0] = vram_bmpload_mem(buf);
		free(buf);
		return spr;
	} else if(buf[0] != 'S' || buf[1] != 'P' || *(short *)(buf + 2) != 0x0201) {
		console_outputwf(
			_T("    spr_load : unsupported file %s Code:%c%c Magic:%02x\n"),
			file,buf[0],buf[1],*(short *)(buf + 2)
		);
		free(spr); free(buf); return NULL;
	}
	p = buf + 4;
	spr->frames = *(int *)p; p += 4;
	spr->frame  = (struct vram_data**)calloc(spr->frames,sizeof(struct vram_data*));
	frame_size  = (struct vram_size*) calloc(spr->frames,sizeof(struct vram_size));
	frame_data  = (unsigned char **)  calloc(spr->frames,sizeof(char *));
	for(i = 0;i < spr->frames; i++) {
		char *data_p;
		int src_len , dec_len;
		frame_size[i].size_x = *(short *)p; p+=2;
		frame_size[i].size_y = *(short *)p; p+=2;
		src_len = *(short *)p; p += 2;
		dec_len = frame_size[i].size_x * frame_size[i].size_y;
		// + 256 ̂̓I[o[t[΍
		frame_data[i] = (unsigned char*)malloc(dec_len + 256);
		data_p = frame_data[i];
		while(dec_len > 0 && src_len > 0) {
			unsigned char value = *(p++);
			if(value == 0) {
				unsigned char count = *(p++);
				dec_len -= count;
				while(count--) { *(data_p++) = value; }
				src_len -= 2;
			} else {
				*(data_p++) = value;
				dec_len--;
				src_len--;
			}
		}
		if(dec_len < 0 || src_len > 0) {
			int j;
			console_outputf("spr_load : frame %d seems broken.\n",i);
			for(j = 0; j <= i; j++) {
				free(frame_data[j]);
			}
			free(frame_data);
			free(frame_size);
			free(buf);
			spr_free(spr);
			return NULL;
		}
	}
	if(size != p - buf + 1024) {
		console_outputf("spr_load : color table size error.\n");
		for(i = 0; i < spr->frames; i++) {
			free(frame_data[i]);
		}
		free(frame_data);
		free(frame_size);
		free(buf);
		spr_free(spr);
		return NULL;
	} else {
		PIXEL color_table[256];
		for(i = 0;i < 256; i++) {
			color_table[i] = vram_RGBA(p[0],p[1],p[2],p[3]);
			p += 4;
		}
		for(i = 0; i < spr->frames; i++) {
			PIXEL c = color_table[frame_data[i][0]];
			color_table[frame_data[i][0]] |= vram_RGBA(0,0,0,255);
			spr->frame[i] = vram_load_color_table(
				frame_data[i],
				frame_size[i].size_x,
				frame_size[i].size_y,
				color_table
			);
			color_table[frame_data[i][0]] = c;
			free(frame_data[i]);
		}
		free(frame_data);
		free(frame_size);
		free(buf);
		return spr;
	}
}

void spr_free(struct spr_data* spr) {
	int i;
	if(spr == NULL) return;
	if(spr->frame) {
		for(i = 0;i < spr->frames ; i++) {
			vram_free(spr->frame[i]);
		}
		free(spr->frame);
	}
	free(spr);
}
