#ifndef _VRAM_H_
#define _VRAM_H_

#include <windows.h>

typedef unsigned long PIXEL;
#define vram_msakA 0xFF000000
#define vram_maskR 0x00FF0000
#define vram_maskG 0x0000FF00
#define vram_maskB 0x000000FF
#define vram_A(c) ((c & vram_msakA) >> 24)
#define vram_R(c) ((c & vram_maskR) >> 16)
#define vram_G(c) ((c & vram_maskG) >>  8)
#define vram_B(c) (c & vram_maskB)
#define vram_RGB(r,g,b)    (PIXEL)(            (r) << 16 | (g) << 8 | (b)) 
#define vram_RGBA(r,g,b,a) (PIXEL)((a) << 24 | (r) << 16 | (g) << 8 | (b)) 
struct vram_size {
	int size_x;
	int size_y;
};

HDC    vram_gethdc(struct vram_data *vram);
struct vram_data* vram_alloc(int x,int y);
struct vram_size* vram_getsize(struct vram_data* vram,struct vram_size* size);
void vram_bitmapinfo(BITMAPINFOHEADER *head,int x,int y,int depth);
struct vram_data* vram_bmpload_mem(unsigned char *buf);
struct vram_data* vram_bmpload_file(struct io_data *io,wchar_t *file);
void vram_free(struct vram_data* vram);

int vram_copy(struct vram_data* from,struct vram_data* to,RECT *from_r,RECT *to_r,int angle,int alpha);
struct vram_data* vram_resample(struct vram_data* from);
void vram_drawtext(struct vram_data* vram,char *data,int x,int y,PIXEL color);

PIXEL vram_getpixel(struct vram_data* vram,int x,int y);
void vram_setpixel(struct vram_data* vram,int x,int y,PIXEL color);
void vram_square(struct vram_data* vram,int x,int y,int xe,int ye,PIXEL color);
struct vram_data *vram_load_color_table(unsigned char *data,int size_x,int size_y,PIXEL *ctbl);

// load save functions
int vram_bmpsave(struct vram_data* vram,struct io_data* io,wchar_t *filename,int depth);
int vram_bmpsave_start(struct vram_data* vram,struct io_data* io,wchar_t *filename,int sizey,int depth);
int vram_bmpsave_data(struct vram_data* vram,int sizey);
int vram_bmpsave_end(struct vram_data* vram);

// avi supports
void vram_aviwrite_start(struct vram_data* vram,struct io_data* io,wchar_t *filename,int interval,int depth);
int  vram_aviwrite_frame(struct vram_data* vram);
void vram_aviwrite_close(struct vram_data* vram);

#endif
